@@ -107,14 +107,13 @@ Sign it, and the certificate is ready!
 
 =head1 USER CERTIFICATE SIGNING REQUEST
 
-We are going to play by the rules, and generate the user's certificate from
-public data only; here, using a PKCS#10 Certificate Signing Request
-(L<Crypt::OpenSSL::CA::AlphabetSoup/CSR>;
+We are going to play by the rules, and not let the CA know the user's
+private key (it will just be saved to disk under the name
+privkey.pem).  Instead, we load the public key from a standard PKCS#10
+Certificate Signing Request (L<Crypt::OpenSSL::CA::AlphabetSoup/CSR>;
 L<Crypt::OpenSSL::CA::AlphabetSoup/SPKAC> is also supported).  Again
-L<Crypt::OpenSSL::CA> cannot fabricate PKCS#10's directly, but you could create
-one with something like
-
-  openssl req -nodes -batch -newkey rsa:1024 -keyout userkey.pem -out user.p10
+L<Crypt::OpenSSL::CA> cannot fabricate PKCS#10's directly, but they
+are easy enough to create with the openssl command line.
 
 Standards-savvy readers know that PKCS#10 requests can contain other
 fields than the public key.  However, L<Crypt::OpenSSL::CA> ignores
@@ -123,20 +122,8 @@ them entirely.
 =cut
 
 {
-  my $user_csr_as_text = <<"PKCS10";
------BEGIN CERTIFICATE REQUEST-----
-MIIBhDCB7gIBADBFMQswCQYDVQQGEwJBVTETMBEGA1UECBMKU29tZS1TdGF0ZTEh
-MB8GA1UEChMYSW50ZXJuZXQgV2lkZ2l0cyBQdHkgTHRkMIGfMA0GCSqGSIb3DQEB
-AQUAA4GNADCBiQKBgQCwoel30tZE9ItO0wfQWx3jGFpMLo41iFhFrqlweTJ7iacM
-bq58tmpDjEONxhqLkNzm05nb2pylskWzKwLQ9NXvchkzK31HKyp89thiVL7ILClV
-YRYMz4QLeB75W+xl6q2pcClQ3NrN7CrR9czvmVFOXNWKWxyQXYi2Ad0qVvNF+wID
-AQABoAAwDQYJKoZIhvcNAQEFBQADgYEAFL1txli+LGSS4V1sVSRdMh054QVk9TKY
-50HTKYR44aCax3fDcnp4H7jR5QEHX0TeHCC5cr8cDDWLEYmCb0UBXr70czrap3n2
-Du3EgKJUHSURsNbkSHSBKupLrw9Ygmipl4vvHRAX59Bqbz4LGEhALnx0eiwK1TtQ
-mk7h7g7cYc8=
------END CERTIFICATE REQUEST-----
-PKCS10
-
+  my $user_csr_as_text =
+    `openssl req -nodes -batch -newkey rsa:1024 -keyout privkey.pem`;
   my $user_pubkey = Crypt::OpenSSL::CA::PublicKey->
     validate_PKCS10($user_csr_as_text);
 
@@ -178,8 +165,8 @@ UTF-8 support.  The validity period (notBefore and notAfter) can be of
 arbitrary size, and transition from utcTime to generalizedTime is
 handled properly.  SubjectAltNames are fully supported regardless of
 type and multiplicity; ditto for certificate policies.  The signature
-algorithm is RSA and the hash can be set to any of OpenSSL's supported
-hashes.  OpenSSL's algorithm for RSA key fingerprints (also known as
+algorithm is RSA and the hash can be set to MD5 (but don't!), SHA1 or
+SHA256.  OpenSSL's algorithm for RSA key fingerprints (also known as
 X509 KeyIDs) is available (but not mandatory) for the subject and
 authority key identifiers.
 
@@ -226,11 +213,7 @@ authority key identifiers.
   $user_cert->set_extension
     (subjectAltName => 'email:johndoe@example.com,email:johndoe@example.net');
 
-  my $fancy_digest_alg = "ripemd160";  # I'd use "sha256" myself, but
-  # some old builds of OpenSSL don't have it.
-  warn "And here is a certificate using $fancy_digest_alg as the digest!\n";
-
-  our $user_cert_as_text = $user_cert->sign($ca_privkey, $fancy_digest_alg);
+  our $user_cert_as_text = $user_cert->sign($ca_privkey, "sha256");
   print $user_cert_as_text;
 }
 
